#!/usr/bin/perl
#               -------------
#               Plebius! Webtools
#               -------------
#           Set of internet utilities
#
#         File: whois.cgi
#  Description: Perform a whois query to determine availability of
#               a domain name.  See the readme.txt for a list of
#               supported Top Level Domains.
#       Author: Martin Kretzmann
#        Email: martin@plebius.org
#          Web: http://www.plebius.org
#      Version: 2.1
#  Copyright : 1999-2000 Martin Kretzmann.  All Rights Reserved.
#      License: GNU General Public License. 
#               http://www.plebius.org/cgi-bin/article.pl?aid=000063
#
#====================================================================

require 'output.cgi';

&main;

sub main {
    ($query, $action) = &data_parse;
    $command = "Whois Query";
    $port = "43";
    my @temp = split(/:/, $query);
    $input = $temp[0];
    $server = $temp[1];
    $ts = 1 if $temp[1];
    @pts = split(/\./, $input);
    if($pts[0] eq 'www' || $#pts > 2)
    {
       &show("","",$command,$input,"Only certain top level domains are supported by this script.  \nDo not include www when submitting your request","Whois ");
       exit;
    }

    $nnum = $#pts;
    $domain = lc($pts[$nnum]);

    # whois servers, updates from http://www.apnic.net/maps/tld-list.html
    %ext = (
           'ad','whois.ripe.net',
           'af','whois.nic.af',
           'al','whois.ripe.net',
           'am','whois.ripe.net',
           'as','whois.nic.as',
           'at','whois.ripe.net',
           'au','whois.aunic.net',
           'az','whois.ripe.net',
           'ba','whois.ripe.net',
           'be','whois.ripe.net',
           'bg','whois.ripe.net',
           'bm','rwhois.bermudanic.bm',
           'br','whois.nic.br',
           'bt','whois.nic.bt',
           'by','whois.ripe.net',
           'ca','whois.cdnnet.ca',
           'cc','whois.nic.cc',
           'ch','whois.nic.ch',
           'ck','whois.ck-nic.org.ck',
           'cn','whois.cnnic.net.cn',
           'com','whois.crsnic.net',
           'cx','whois.nic.cx',
           'cy','whois.ripe.net',
           'cz','whois.ripe.net',
           'de','whois.ripe.net',
           'dk','whois.ripe.net',
           'dz','whois.ripe.net',
           'ee','whois.ripe.net',
           'edu','whois.crsnic.net',
           'eg','whois.ripe.net',
           'es','whois.ripe.net',
           'fi','whois.ripe.net',
           'fo','whois.ripe.net',
           'fr','whois.ripe.net',
           'ga','whois.ripe.net',
           'gb','whois.ripe.net',
           'ge','whois.ripe.net',
           'gf','whois.nplus.gf',
           'gi','whois.nic.gi',
           'gl','whois.ripe.net',
           'gm','whois.ripe.net',
           'gov','whois.nic.gov',
           'gr','whois.ripe.net',
           'gs','whois.adamsnames.tc',
           'hk','whois.hknic.net.hk',
           'hm','whois.nic.hm',
           'hr','whois.ripe.net',
           'hu','whois.ripe.net',
           'id','whois.idnic.net.id',
           'il','whois.ripe.net',
           'in','whois.ncst.ernet.in',
           'io','whois.io',
           'is','whois.ripe.net',
           'it','whois.ripe.net',
           'jo','whois.ripe.net',
           'jp','whois.nic.ad.jp',
           'kr','whois.krnic.net',
           'kz','whois.domain.kz',
           'li','whois.nic.li',
           'lk','whois.nic.lk',
           'lt','whois.ripe.net',
           'lu','whois.ripe.net',
           'lv','whois.ripe.net',
           'ly','whois.nic.ly',
           'ma','whois.ripe.net',
           'mc','whois.ripe.net',
           'md','whois.ripe.net',
           'mil','whois.nic.mil',
           'mk','whois.ripe.net',
           'mm','whois.nic.mm',
           'ms','whois.adamsnames.tc',
           'mt','whois.ripe.net',
           'mx','whois.nic.mx',
           'net','whois.crsnic.net',
           'nl','www.domain-registry.nl',
           'no','whois.ripe.net',
           'nu','whois.nic.nu',
           'nz','whois.patho.gen.nz',
           'org','whois.crsnic.net',
           'pe','rwhois.rcp.net.pe',
           'pk','whois.pknic.net.pk',         
           'pl','whois.ripe.net',
           'pr','whois.uprr.pr',
           'pw','whois.nic.pw',
           'ro','whois.ripe.net',
           'ru','whois.ripn.net',
           'se','whois.nic-se.se',
           'sg','whois.nic.net.sg',
           'sh','whois.nic.sh',
           'si','whois.ripe.net',
           'sk','whois.ripe.net',
           'sm','whois.ripe.net',
           'so','whois.nic.so',
           'st','whois.nic.st',
           'tc','whois.adamsnames.tc',
           'tf','whois.adamsnames.tc',
           'th','whois.thnic.net',
           'tj','whois.nic.tj',
           'tm','whois.nic.tm',
           'tn','whois.ripe.net',
           'to','whois.tonic.to',
           'tr','whois.ripe.net',
           'tw','whois.twnic.net',
           'ua','whois.ripe.net',
           'uk','whois.nic.uk',
           'us','whois.isi.edu',
           'va','whois.ripe.net',
           've','rwhois.reacciun.ve',
           'vg','whois.adamsnames.tc',
           'yu','whois.ripe.net',
           'za','whois.frd.ac.za',
           );

        $server = $ext{$domain} if ($ext{$domain} && !$server);
        # default server
        $server = 'whois.crsnic.net' if $server eq '';

        # added support for .co.za domains
        if ($input =~ /\.co\.za$/i)
           {
                   $server = 'co.za';
                   $port   = 80;
                   $input = "GET /cgi-bin/whois.sh?Domain=$input";
                   &IOSocket;
                   $output = join("THIS IS A DELIMITER", @output);
                   $output =~ s/<([^>]*)>//ig;
                   $output =~ s/\n+/\n/ig;
                   @output = split(/THIS IS A DELIMITER/, $output);
           } 
        # added support for correct .ie domains
        elsif ($input =~ /\.ie$/i)
           {
                $server = "www.domainregistry.ie";
                $port   = 80;
                $input = "GET /cgi-bin/dns-search.cgi?foo=foo&domain=$input";
                &IOSocket;
                $output = join("DELIMITER!", @output);
                $output =~ s/\s+/ /g;
                $output =~ s/<br>/\n/ig;
                $output =~ s/<p>/\n\n/ig;
                $output =~ s/\n\n*/\n/g;
                $output =~ s/<([^>]*)>//ig;
                @output = split(/DELIMITER!/, $output);
           }
        else 
           {
                   &IOSocket;
           }

        if (($domain eq 'com' || $domain eq 'net' || $domain eq 'org' || $domain eq 'edu') && ($ts != 1))
        {
            my (@INFO) = @output;
            foreach $line(@INFO)
            {
                    if ($line =~ /Registrar:/i)
                    {
                        my @tmp = split(/:/, $line);
                        $registrar = $tmp[1];
                        $available = "This domain has already been registered.\nDisplaying domain information provided by $registrar\n";
                    }
                    if ($line =~ /Whois Server:/i)
                    {
                       my @temp = split(/:/, $line);
                       $nextserv = $temp[1];
                       chomp($nextserv);
                       $nextserv =~ s/\s+//ig;
                       last;
                    }                           
            }
            if($nextserv)
            {
               $server = $nextserv;
               &IOSocket;
            }
            else 
            {
               $available = "This domain is available\n";
            } 
        }
        $output = "Connected to $server\n$available".join(" ", @output)."\nUse the following syntax if you wish to specify which whois\nserver will handle your request.\t query:server";
        $stop = &gettime;
        &show($start,$stop,$command,$query,$output,"Whois ");
}

sub IOSocket{
        use IO::Socket;
        $start = &gettime;
        $next = "\015\012";
        $socket = IO::Socket::INET->new(Proto=>"tcp",
             PeerAddr=>$server,PeerPort=>$port,);

        unless ($socket) { $stop = &gettime; die &show($start,$stop,$command,$input,"$@",$special); }

        print $socket "$input" . $next;
        @output = <$socket>;
        close $socket;
}
